@@ -25,6 +25,7 @@ from ..types import data_type
 from ..types import general_config_type
 from ..configs import _parse_config
 from ..trainer import Trainer
+from ..trainer import TrainerCallback
 from ..trainer import IntermediateResults
 from ..pipeline import Pipeline
 from ..protocol import DataProtocol
@@ -33,6 +34,8 @@ from ..misc._api import _make_saving_path
 from ..misc._api import _fetch_saving_paths
 from ..misc.toolkit import to_2d
 from ..misc.toolkit import inject_mlflow_stuffs
+from ..models.base import model_dict
+from ..models.base import PipeConfig
 
 
 def make(
@@ -47,6 +50,74 @@ def make(
     return Pipeline.make(parsed_config, parsed_increment_config)
 
 
+class _PipeConfigManager:
+    def __init__(
+        self,
+        pipes: Dict[str, PipeConfig],
+        meta_configs: Dict[str, Dict[str, Any]],
+        pipe: str,
+    ):
+        self.pipe_config = pipes[pipe]
+        self.meta_config = meta_configs.setdefault(pipe, {})
+        self.pipes, self.pipe = pipes, pipe
+
+    @property
+    def transform_config(self) -> Dict[str, Any]:
+        return self.meta_config.setdefault("transform", {})
+
+    @transform_config.setter
+    def transform_config(self, value: Dict[str, Any]) -> None:
+        self.meta_config["transform"] = value
+
+    @property
+    def extractor_config(self) -> Dict[str, Any]:
+        return self.meta_config.setdefault("extractor", {})
+
+    @extractor_config.setter
+    def extractor_config(self, value: Dict[str, Any]) -> None:
+        self.meta_config["extractor"] = value
+
+    @property
+    def head_config(self) -> Dict[str, Any]:
+        return self.meta_config.setdefault("head", {})
+
+    @head_config.setter
+    def head_config(self, value: Dict[str, Any]) -> None:
+        self.meta_config["head"] = value
+
+    def replace(self, **kwargs: Any) -> None:
+        pipe_config_dict = self.pipe_config._asdict()
+        pipe_config_dict.update(kwargs)
+        self.pipe_config = PipeConfig(**pipe_config_dict)
+        self.pipes[self.pipe] = self.pipe_config
+
+
+class ModelConfig:
+    def __init__(self, name: str):
+        self.name = name
+        base = model_dict[name]
+        self.registered_pipes = base.registered_pipes
+        self.registered_meta_configs = base.registered_meta_configs or {}
+        base.registered_meta_configs = self.registered_meta_configs
+
+    def switch(self, pipe: Optional[str] = None) -> _PipeConfigManager:
+        if self.registered_pipes is None:
+            raise ValueError(f"no pipes are registered for {self.name}")
+        if pipe is None:
+            if len(self.registered_pipes) > 1:
+                pipe_str = ", ".join(sorted(self.registered_pipes))
+                raise ValueError(
+                    "`pipe` must be specified when more than one pipe is registered "
+                    f"(current registered pipes: {pipe_str})"
+                )
+            pipe = list(self.registered_pipes.keys())[0]
+        return _PipeConfigManager(
+            self.registered_pipes,
+            self.registered_meta_configs,
+            pipe,
+        )
+
+
 def make_from(
     identifier: str = "cflearn",
     saving_folder: Optional[str] = None,
@@ -507,8 +578,13 @@ def make_toy_model(
     return make(**updated).fit(*data_tuple)
 
 
+def switch_trainer_callback(callback_base: Type[TrainerCallback]) -> None:
+    Trainer.callback_base = callback_base
+
+
 __all__ = [
     "make",
+    "ModelConfig",
     "make_from",
     "finetune",
     "save",
@@ -518,6 +594,7 @@ __all__ = [
     "load_experiment_results",
     "repeat_with",
     "make_toy_model",
+    "switch_trainer_callback",
     "Task",
     "Experiment",
     "ModelPattern",
