@@ -1,5 +1,6 @@
 import os
 import json
+import math
 import torch
 import shutil
 
@@ -8,6 +9,7 @@ import matplotlib.pyplot as plt
 import matplotlib.patches as mpatch
 
 from typing import *
+from tqdm import tqdm
 from cfdata.tabular import TabularDataset
 from cftool.ml import ModelPattern
 from cftool.misc import show_or_save
@@ -35,6 +37,7 @@ from .inference import PreProcessor
 from .misc._api import _fetch_saving_paths
 from .misc.toolkit import to_2d
 from .misc.toolkit import to_relative
+from .misc.toolkit import eval_context
 from .misc.toolkit import LoggingMixinWithRank
 from .misc.time_series import TSLabelCollator
 from .models.base import model_dict
@@ -140,6 +143,12 @@ class Pipeline(LoggingMixinWithRank):
             self.shuffle_tr,
             self.tr_weights,
         )
+
+        if self.batch_size is None:  # type: ignore
+            self.batch_size = max(128, min(1024, int(round(0.01 * len(self.tr_data)))))
+            self.batch_size = 2 ** int(round(math.log2(self.batch_size)))
+            self.config["lr_ratio"] = math.log2(self.batch_size / 128)
+
         self.tr_loader = DataLoaderProtocol.make(
             self.loader_protocol,
             self.batch_size,
@@ -828,5 +837,17 @@ class Pipeline(LoggingMixinWithRank):
                     trainer.final_results = IntermediateResults(*json.load(f))
         return pipeline
 
+    def profile_forward(self, *, num_repeat: int = 100, **kwargs: Any) -> None:
+        if self.model is None:
+            raise ValueError("`model` is not generated yet")
+        input_sample = {k: v.to("cpu") for k, v in self.model.input_sample.items()}
+        self.model.to("cpu")
+        with eval_context(self.model):
+            with torch.autograd.profiler.profile(**kwargs) as prof:
+                for _ in tqdm(range(num_repeat), total=num_repeat):
+                    self.model(input_sample)
+        print(prof.key_averages().table(sort_by="self_cpu_time_total"))
+        self.model.to(self.device)
+
 
 __all__ = ["Pipeline"]
