@@ -320,6 +320,10 @@ class DataLoaderProtocol(ABC):
     def copy(self) -> "DataLoaderProtocol":
         pass
 
+    @property
+    def num_samples(self) -> int:
+        return len(self.data)
+
     @classmethod
     def get(cls, name: str) -> Type["DataLoaderProtocol"]:
         return loader_dict[name]
@@ -510,7 +514,7 @@ class TrainerState:
 
     @property
     def should_start_snapshot(self) -> bool:
-        return self.step >= self.snapshot_start_step and self.epoch > self.min_epoch
+        return self.step >= self.snapshot_start_step
 
     @property
     def should_start_monitor_plateau(self) -> bool:
@@ -639,7 +643,6 @@ class ModelProtocol(nn.Module, LoggingMixinWithRank, metaclass=ABCMeta):
         self,
         folder: str,
         strict: bool = True,
-        deepspeed: bool = False,
         state_dict_callback: Optional[Callable[[Dict[str, Any]], None]] = None,
     ) -> bool:
         checkpoints = self.sorted_checkpoints(folder)
@@ -651,22 +654,21 @@ class ModelProtocol(nn.Module, LoggingMixinWithRank, metaclass=ABCMeta):
             )
             return False
         success = False
-        if not deepspeed:
-            for checkpoint in checkpoints:
-                model_file = os.path.join(folder, checkpoint)
-                if not os.path.isfile(model_file):
-                    continue
-                self.log_msg(  # type: ignore
-                    f"restoring from {model_file}",
-                    self.info_prefix,  # type: ignore
-                    4,
-                )
-                states = torch.load(model_file, map_location=self.device)
-                if state_dict_callback is not None:
-                    state_dict_callback(states)
-                self.load_state_dict(states, strict)
-                success = True
-                break
+        for checkpoint in checkpoints:
+            model_file = os.path.join(folder, checkpoint)
+            if not os.path.isfile(model_file):
+                continue
+            self.log_msg(  # type: ignore
+                f"restoring from {model_file}",
+                self.info_prefix,  # type: ignore
+                4,
+            )
+            states = torch.load(model_file, map_location=self.device)
+            if state_dict_callback is not None:
+                state_dict_callback(states)
+            self.load_state_dict(states, strict)
+            success = True
+            break
         return success
 
     def step(
@@ -778,7 +780,9 @@ class InferenceProtocol(ABC):
         *,
         use_tqdm: bool = False,
         return_loss: bool = True,
+        return_outputs: bool = True,
         state: Optional[TrainerState] = None,
+        portion: float = 1.0,
         **kwargs: Any,
     ) -> InferenceOutputs:
         labels_key = loader.loader.labels_key
@@ -786,10 +790,12 @@ class InferenceProtocol(ABC):
             loader = self.to_tqdm(loader)
 
         def _core() -> InferenceOutputs:
-            results: Dict[str, List[np.ndarray]] = {}
+            results: Dict[str, Optional[List[np.ndarray]]] = {}
             loss_items: Dict[str, List[float]] = {}
             labels = []
             for i, (batch, batch_indices) in enumerate(loader):
+                if i / len(loader) >= portion:
+                    break
                 local_labels = batch[labels_key]
                 if local_labels is not None:
                     if not isinstance(local_labels, np.ndarray):
@@ -801,6 +807,7 @@ class InferenceProtocol(ABC):
                 else:
                     assert self.model is not None
                     with eval_context(self.model, use_grad=use_grad):
+                        assert not self.model.training
                         local_kwargs = shallow_copy_dict(kwargs)
                         local_kwargs["return_loss"] = return_loss
                         local_results = self.model(
@@ -815,6 +822,7 @@ class InferenceProtocol(ABC):
                         local_losses = None
                     else:
                         with eval_context(self.model, use_grad=use_grad):
+                            assert not self.model.training
                             local_losses = self.model.loss_function(
                                 i,
                                 batch,
@@ -829,13 +837,19 @@ class InferenceProtocol(ABC):
                         v_np = v
                     else:
                         v_np = to_numpy(v)
-                    results.setdefault(k, []).append(v_np)
+                    if not return_outputs:
+                        results[k] = None
+                    else:
+                        results.setdefault(k, []).append(v_np)  # type: ignore
                 if local_losses is not None:
                     for k, v in local_losses.items():
                         loss_items.setdefault(k, []).append(v.item())
 
+            if return_outputs:
+                results = {k: np.vstack(v) for k, v in results.items()}
+
             return InferenceOutputs(
-                {k: np.vstack(v) for k, v in results.items()},
+                results,
                 None
                 if not loss_items
                 else {k: sum(v) / len(v) for k, v in loss_items.items()},
@@ -929,6 +943,7 @@ class InferenceProtocol(ABC):
             return_all,
             requires_recover,
             returns_probabilities,
+            **shallow_copy_dict(kwargs),
         )
 
     def predict_with(self, probabilities: np.ndarray) -> np.ndarray:
