@@ -271,8 +271,9 @@ class TrainMonitor:
             )
             return True
         if self.info["save_checkpoint"]:
-            self.log_msg(f"{self.info['info']}", self.monitored.info_prefix, 3)
-            self.monitored.on_save_checkpoint(new_score)
+            if self.monitored.is_rank_0:
+                self.log_msg(f"{self.info['info']}", self.monitored.info_prefix, 3)
+                self.monitored.on_save_checkpoint(new_score)
         if self.state.should_extend_epoch:
             if self.is_lazy:
                 return True
@@ -599,8 +600,8 @@ class Trainer(MonitoredMixin):
             "min_lr": 1.0e-8,
             "verbose": self._verbose_level >= 3,
             "patience": max(
-                10,
-                self.state.snapshot_start_step // self.state.num_step_per_snapshot,
+                10 * self.state.num_step_per_snapshot,
+                self.state.snapshot_start_step,
             ),
         }
         return {
@@ -634,6 +635,8 @@ class Trainer(MonitoredMixin):
         optimizers_settings = self.config.setdefault("optimizers", {"all": {}})
         self.optimizers: Dict[str, Optimizer] = {}
         self.schedulers: Dict[str, Optional[_LRScheduler]] = {}
+        lr_ratio = max(1.0, self.environment.pipeline_config.get("lr_ratio", 1.0))
+        default_lr = 1.0e-3 * lr_ratio
         for params_name, opt_setting in optimizers_settings.items():
             optimizer = opt_setting["optimizer"]
             optimizer_config = opt_setting["optimizer_config"]
@@ -641,22 +644,20 @@ class Trainer(MonitoredMixin):
             scheduler_config = opt_setting["scheduler_config"]
             # we need to consider warmup here because it will modify the `lr`
             if scheduler != "warmup":
-                optimizer_config.setdefault("lr", 1e-3)
+                optimizer_config.setdefault("lr", default_lr)
             else:
                 multiplier = scheduler_config.setdefault("multiplier", 3)
-                optimizer_config.setdefault("lr", multiplier * 1.0e-3)
+                optimizer_config.setdefault("lr", default_lr / multiplier)
+                default_max_warmup_step = int(round(3.0e5 / self.tr_loader.batch_size))
+                num_step_per_epoch = self.state.num_step_per_epoch
                 warmup_step = scheduler_config.setdefault(
                     "warmup_step",
-                    10 * self.state.num_step_per_epoch,
+                    min(default_max_warmup_step, 10 * num_step_per_epoch),
                 )
-                self.state.plateau_start += int(
-                    warmup_step / self.state.num_step_per_snapshot
-                )
-                if self.state._snapshot_start_step is not None:
-                    self.state._snapshot_start_step += warmup_step
-                else:
-                    self.state.min_num_sample += self.tr_loader.batch_size * warmup_step
-                optimizer_config["lr"] /= multiplier
+                warmup_snapshot = int(warmup_step / self.state.num_step_per_snapshot)
+                warmup_epoch = math.floor(warmup_step / num_step_per_epoch)
+                self.state.plateau_start += warmup_snapshot
+                self.state.min_epoch += warmup_epoch
             # the default settings of optimizers
             if optimizer == "nag":
                 optimizer_config.setdefault("momentum", 0.999)
@@ -671,21 +672,15 @@ class Trainer(MonitoredMixin):
             # scheduler
             default_lr_configs = self.default_lr_configs(opt, optimizer_config)
             default_lr_config = default_lr_configs.get(scheduler)
-            error_msg = f"default scheduler config for {scheduler} is not specified"
             if default_lr_config is not None:
                 scheduler_config = update_dict(scheduler_config, default_lr_config)
-            else:
-                if scheduler != "warmup":
-                    raise ValueError(error_msg)
             if scheduler == "warmup":
                 sab = scheduler_config.get("scheduler_afterwards_base", "plateau")
                 if sab == "warmup":
                     raise ValueError("warmup should not be used inside a warmup")
                 sac = scheduler_config.get("scheduler_afterwards_config", {})
                 default_lr_config = default_lr_configs.get(sab)
-                if default_lr_config is None:
-                    raise ValueError(error_msg)
-                sac = update_dict(sac, default_lr_config)
+                sac = update_dict(sac, default_lr_config or {})
                 sab = scheduler_dict[sab]
                 scheduler_config["scheduler_afterwards_base"] = sab
                 scheduler_config["scheduler_afterwards_config"] = sac
@@ -736,7 +731,9 @@ class Trainer(MonitoredMixin):
             else:
                 metric_key = f"{metric_type}_config"
                 sub_metric_config = metric_config.setdefault(metric_key, {})
-                self.metrics[metric_type] = Metrics(metric_type, sub_metric_config)
+                metric = Metrics(metric_type, sub_metric_config)
+                self.metrics[metric_type] = metric
+                metric.trainer = self
             if not 0.0 < metric_decay < 1.0:
                 self.metrics_decay = None
             else:
@@ -803,14 +800,15 @@ class Trainer(MonitoredMixin):
     ) -> Tuple[bool, Optional[Dict[str, Any]]]:
         kwargs = {}
         should_log_lr = self.state.should_log_lr
-        if key in self.schedulers_requires_metric and not (
-            isinstance(scheduler, WarmupScheduler) and not scheduler.finished_warmup
-        ):
-            if self.intermediate is None or not self.intermediate_updated:
-                return should_log_lr, None
-            kwargs["metrics"] = self.intermediate.final_score
+        is_warmup = isinstance(scheduler, WarmupScheduler)
+        requires_metric = key in self.schedulers_requires_metric
+        if requires_metric and not (is_warmup and not scheduler.finished_warmup):
+            if self.intermediate is None:
+                kwargs["metrics"] = -math.inf
+            else:
+                kwargs["metrics"] = self.intermediate.final_score
+            should_log_lr = self.intermediate_updated
             self.intermediate_updated = False
-            should_log_lr = True
         return should_log_lr, kwargs
 
     def _scheduler_step(self) -> None:
@@ -915,7 +913,8 @@ class Trainer(MonitoredMixin):
             with timing_context(self, "EMA", enable=self.timing):
                 self.model.apply_ema()
         if self.state.should_log_losses:
-            self._log_scalars(step_outputs.loss_items)
+            tr_losses = {f"tr_{k}": v for k, v in step_outputs.loss_items.items()}
+            self._log_scalars(tr_losses)
 
     # core step on each epoch
     def _step(
@@ -969,7 +968,9 @@ class Trainer(MonitoredMixin):
             loss = step_outputs.loss_dict["loss"]
             engine.backward(loss)
         with timing_context(self, "ds.step", enable=self.timing):
-            engine.step()
+            scheduler = list(self.schedulers.values())[0]
+            _, kwargs = self._get_scheduler_settings("all", scheduler)
+            engine.step(lr_kwargs=kwargs)
         return step_outputs
 
     # api
@@ -1004,6 +1005,8 @@ class Trainer(MonitoredMixin):
         self._init_metrics()
         # monitor
         monitor_config = self.config.setdefault("monitor_config", {})
+        default_patience = max(4, math.ceil(math.log10(tr_loader.num_samples)))
+        monitor_config.setdefault("patience", default_patience)
         self._monitor = TrainMonitor.monitor(self, **monitor_config)
         # train
         self.model.info()
@@ -1073,6 +1076,7 @@ class Trainer(MonitoredMixin):
         binary_outputs: Optional[InferenceOutputs] = None,
         loader: Optional[PrefetchLoader] = None,
         loader_name: Optional[str] = None,
+        metrics_kwargs: Optional[Dict[str, Dict[str, Any]]] = None,
     ) -> Tuple[InferenceOutputs, IntermediateResults]:
         if self.cv_loader is None and self.tr_loader._num_siamese > 1:
             raise ValueError("cv set should be provided when num_siamese > 1")
@@ -1101,6 +1105,7 @@ class Trainer(MonitoredMixin):
             results = self.inference.predict_from_outputs(
                 outputs,
                 return_all=True,
+                requires_recover=False,
                 returns_probabilities=False,
             )
             probabilities = None
@@ -1139,7 +1144,12 @@ class Trainer(MonitoredMixin):
                                 metric_predictions = probabilities
                             else:
                                 metric_predictions = to_prob(logits)
-                sub_metric = metric_ins.metric(labels, metric_predictions)
+                metric_kwargs = (metrics_kwargs or {}).get(metric_type)
+                sub_metric = metric_ins.metric(
+                    labels,
+                    metric_predictions,
+                    **shallow_copy_dict(metric_kwargs or {}),
+                )
                 metrics[metric_type] = float(sub_metric)
             if self.metrics_decay is not None and self.state.should_start_snapshot:
                 use_decayed = True
@@ -1166,30 +1176,16 @@ class Trainer(MonitoredMixin):
     def save_checkpoint(self, score: float, folder: Optional[str] = None) -> None:
         if folder is None:
             folder = self.checkpoint_folder
-        if self.deepspeed:
-            file = fix_float_to_length(score, 8)
-            if os.path.isdir(os.path.join(folder, file)):
-                return None
-            self.model_engines["all"].save_checkpoint(
-                folder,
-                file,
-                client_state={
-                    "step": self.state,
-                    "epoch": self.state.epoch,
-                    "score": score,
-                },
-            )
-        else:
-            # leave top_k snapshots only
-            if self.state.max_snapshot_file > 0:
-                checkpoints = self.model.sorted_checkpoints(folder)
-                if len(checkpoints) >= self.state.max_snapshot_file:
-                    for file in checkpoints[self.state.max_snapshot_file - 1 :]:
-                        self.checkpoint_scores.pop(file)
-                        os.remove(os.path.join(folder, file))
-            # pt
-            file = f"{self.model.pt_prefix}{self.state.epoch}.pt"
-            torch.save(self.model.state_dict(), os.path.join(folder, file))
+        # leave top_k snapshots only
+        if self.state.max_snapshot_file > 0:
+            checkpoints = self.model.sorted_checkpoints(folder)
+            if len(checkpoints) >= self.state.max_snapshot_file:
+                for file in checkpoints[self.state.max_snapshot_file - 1 :]:
+                    self.checkpoint_scores.pop(file)
+                    os.remove(os.path.join(folder, file))
+        # pt
+        file = f"{self.model.pt_prefix}{self.state.epoch}.pt"
+        torch.save(self.model.state_dict(), os.path.join(folder, file))
         # scores
         self.checkpoint_scores[file] = score
         with open(os.path.join(folder, self.model.scores_file), "w") as f:
@@ -1203,12 +1199,7 @@ class Trainer(MonitoredMixin):
     ) -> bool:
         if folder is None:
             folder = self.checkpoint_folder
-        return self.model.restore_checkpoint(
-            folder,
-            strict,
-            self.deepspeed,
-            state_dict_callback,
-        )
+        return self.model.restore_checkpoint(folder, strict, state_dict_callback)
 
 
 __all__ = [
